import datetime
import os

from jinja2 import Environment, FileSystemLoader

from src.constants import ALERT_TYPE_LIKELY, ALERT_TYPE_MDR, ALERT_SOURCE_TYPE_APP, ALERT_SOURCE_TYPE_DEV, ALERT_COUNT, \
    ALERT_TTR
from src.processor import daily_data_processor
from src.processor.daily_data_processor import get_start_and_end_date, get_time_range_in_plot, get_statistic
from src.util.logger import logger
from src.widgets.daily_report_widgets import plot_daily_severity_pie_chart, plot_daily_line_chart, \
    plot_weekly_severity_bars, \
    plot_daily_horizontal_bars


def daily_report_generator(org_data, report_date):
    """
    Convert raw daily statistic data into HTML format.
    :param report_date:
    :param org_data: Statistic data for an organization.
    :return: None
    """
    HERE = os.path.dirname(os.path.abspath(__file__))
    J2 = Environment(loader=FileSystemLoader(HERE), trim_blocks=True)
    formatted_data = {}
    org_key = org_data.get("org_key")
    org_name = org_data.get("org_name")

    report_date = report_date.strftime("%Y-%m-%d")
    logger.info(f"[{org_key}] Extract basic statistic data for the daily report")
    # Basic four statistics value in the first row on the report.
    # PAST DAY
    formatted_data['likely_threats'] = get_statistic(org_data, ALERT_TYPE_LIKELY, ALERT_COUNT, report_date)
    formatted_data['avg_ttr_likely_alerts'] = get_statistic(org_data, ALERT_TYPE_LIKELY,
                                                                                  ALERT_TTR, report_date)
    formatted_data['eligible_for_mdr_alerts'] = get_statistic(org_data, ALERT_TYPE_MDR, ALERT_COUNT, report_date)
    formatted_data['avg_ttr_mdr_alerts'] = get_statistic(org_data, ALERT_TYPE_MDR, ALERT_TTR,
                                                                               report_date)
    # Date range of alerts and assets
    report_start_time, report_end_time = get_start_and_end_date()
    formatted_data['org_name'] = org_name
    formatted_data['start_time_frame'] = report_start_time
    formatted_data['end_time_frame'] = report_end_time
    formatted_data['date_range'] = get_time_range_in_plot()

    logger.info(f"[{org_key}] Generating plots in the daily report")

    # Generate charts for different type of alerts and assets.
    formatted_data['pie_likely_severity'] = plot_daily_severity_pie_chart(org_data, ALERT_TYPE_LIKELY, report_date)
    formatted_data['pie_mdr_severity'] = plot_daily_severity_pie_chart(org_data, ALERT_TYPE_MDR, report_date)

    formatted_data['line_likely_count'] = plot_daily_line_chart(org_data, ALERT_TYPE_LIKELY, ALERT_COUNT)
    formatted_data['line_likely_avg_time'] = plot_daily_line_chart(org_data, ALERT_TYPE_LIKELY, ALERT_TTR)

    formatted_data['line_mdr_count'] = plot_daily_line_chart(org_data, ALERT_TYPE_MDR, ALERT_COUNT)
    formatted_data['line_mdr_avg_time'] = plot_daily_line_chart(org_data, ALERT_TYPE_MDR, ALERT_TTR)

    formatted_data['bar_likely_severity'] = plot_weekly_severity_bars(org_data, ALERT_TYPE_LIKELY, org_key)
    formatted_data['bar_mdr_severity'] = plot_weekly_severity_bars(org_data, ALERT_TYPE_MDR, org_key)

    formatted_data['bar_horizontal_process_likely'] = plot_daily_horizontal_bars(org_data, ALERT_TYPE_LIKELY,
                                                                                 ALERT_SOURCE_TYPE_APP, report_date)
    formatted_data['bar_horizontal_asset_likely'] = plot_daily_horizontal_bars(org_data, ALERT_TYPE_LIKELY,
                                                                               ALERT_SOURCE_TYPE_DEV, report_date)

    formatted_data['bar_horizontal_process_mdr'] = plot_daily_horizontal_bars(org_data, ALERT_TYPE_MDR,
                                                                              ALERT_SOURCE_TYPE_APP, report_date)
    formatted_data['bar_horizontal_asset_mdr'] = plot_daily_horizontal_bars(org_data, ALERT_TYPE_MDR,
                                                                            ALERT_SOURCE_TYPE_DEV, report_date)

    template = J2.get_template('templates/dailyReport/dailyReport.html')
    html = template.render(formatted_data)
    html_file = f'{org_key}_daily_report_generated.html'
    with open(html_file, "w") as file_html:
        file_html.write(html)
    return html_file


def get_daily_report_key(org_key, report_date):
    return f'daily_report/{org_key}/{report_date}.pdf'
