from enum import Enum

from marshmallow import Schema, fields, post_load, ValidationError
from typing import List


class NotificationType(Enum):
    REPORTING = 'REPORTING'

    def __repr__(self):
        return f"Notification type: {self.value}"

    def __str__(self):
        return self.value


def validate_non_empty_list(value):
    if not value:
        raise ValidationError("The list must not be empty.")
    for email in value:
        if not email or not isinstance(email, str):
            raise ValidationError("Each email must be a non-empty string.")


class Email:
    def __init__(self, notification_type: NotificationType,
                 sender: str,
                 recipients: List[str],
                 subject: str,
                 html_content: str,
                 text_body: str,
                 s3_url: str,
                 report_present: bool):
        self.html_content = html_content
        self.text_body = text_body
        self.notification_type = notification_type
        self.recipients = recipients
        self.report_present = report_present
        self.s3_url = s3_url
        self.sender = sender
        self.subject = subject


class EmailSchema(Schema):
    html_content = fields.Str(required=True, data_key='htmlContent')
    text_content = fields.Str(required=True, data_key='textContent')
    notification_type = fields.Str(required=True, data_key='notificationType')
    org_key = fields.Str(required=True, data_key='orgKey')
    recipients = fields.List(fields.Email(), required=True, validate=validate_non_empty_list,
                             data_key='recipientEmailList')
    report_present = fields.Bool(required=True, data_key='reportPresent')
    s3_url = fields.Str(required=True, data_key='s3URI')
    sender = fields.Email(required=True, data_key='senderEmail')
    subject = fields.Str(required=True, data_key='subject')
