import os

from jinja2 import Environment, FileSystemLoader

from src.constants import *
from src.processor.monthly_data_processor import get_asset_stack_plot_data, get_report_year_month, get_fixed_values, \
    get_avg_time_to_determination, \
    get_num_threats, get_plot_date_range, get_start_and_end_date
from src.widgets.monthly_report_widgets import plot_alert_asset_industry_comp_line_plots, plot_asset_stack_area_chart, \
    plot_monthly_horizontal_bars, \
    plot_half_year_severities, plot_monthly_severity_pie_chart, plot_half_year_line_chart, plot_monthly_line_chart
from src.util.logger import logger


def monthly_report_generator(data):
    HERE = os.path.dirname(os.path.abspath(__file__))
    J2 = Environment(loader=FileSystemLoader(HERE), trim_blocks=True)
    formatted_data = {}
    org_key = data.get("org_key")
    org_name = data.get("org_name")

    # PAST MONTH
    formatted_data['likely_threats'] = get_num_threats(data, ALERT_TYPE_LIKELY)
    formatted_data['avg_ttr_likely_alerts'] = get_avg_time_to_determination(data, ALERT_TYPE_LIKELY)
    formatted_data['eligible_for_mdr_alerts'] = get_num_threats(data, ALERT_TYPE_MDR)
    formatted_data['avg_ttr_mdr_alerts'] = get_avg_time_to_determination(data, ALERT_TYPE_MDR)
    start_end = get_start_and_end_date()
    formatted_data['org_name'] = org_name
    formatted_data['start_time_frame'] = start_end[0]
    formatted_data['end_time_frame'] = start_end[1]
    formatted_data['date_range'] = get_plot_date_range()
    formatted_data['mdr_eligible_alerts'] = get_num_threats(data, ALERT_TYPE_MDR)
    formatted_data['average_time_to_determination_mdr_eligible_alerts'] = get_avg_time_to_determination(data,
                                                                                                        ALERT_TYPE_MDR)

    logger.info(f"[{org_key}] Generating likely threat plots in the monthly report")

    # LIKELY THREATS
    formatted_data['line_likely_count_monthly'] = plot_monthly_line_chart(data, ALERT_TYPE_LIKELY, ALERT_COUNT)
    formatted_data['line_likely_avg_monthly'] = plot_monthly_line_chart(data, ALERT_TYPE_LIKELY, ALERT_TTR)
    formatted_data['line_likely_count_half_yearly'] = plot_half_year_line_chart()
    formatted_data['line_likely_avg_half_yearly'] = plot_half_year_line_chart()
    formatted_data['pie_likely_severity_monthly'] = plot_monthly_severity_pie_chart(data, ALERT_TYPE_LIKELY)
    formatted_data['bar_likely_severity_half_yearly'] = plot_half_year_severities()
    formatted_data['bar_horizontal_process_likely_monthly'] = plot_monthly_horizontal_bars(data, ALERT_TYPE_LIKELY,
                                                                                           ALERT_SOURCE_TYPE_APP)
    formatted_data['bar_horizontal_asset_likely_monthly'] = plot_monthly_horizontal_bars(data, ALERT_TYPE_LIKELY,
                                                                                         ALERT_SOURCE_TYPE_DEV)

    logger.info(f"[{org_key}] Generating mdr threat plots in the monthly report")

    # MDR ELIGIBLE ALERTS
    formatted_data['line_mdr_count_monthly'] = plot_monthly_line_chart(data, ALERT_TYPE_MDR, ALERT_COUNT)
    formatted_data['line_mdr_avg_monthly'] = plot_monthly_line_chart(data, ALERT_TYPE_MDR, ALERT_TTR)
    formatted_data['line_mdr_count_half_yearly'] = plot_half_year_line_chart()
    formatted_data['line_mdr_avg_half_yearly'] = plot_half_year_line_chart()
    formatted_data['pie_mdr_severity_monthly'] = plot_monthly_severity_pie_chart(data, ALERT_TYPE_MDR)
    formatted_data['bar_mdr_severity_half_yearly'] = plot_half_year_severities()
    formatted_data['bar_horizontal_process_mdr_monthly'] = plot_monthly_horizontal_bars(data, ALERT_TYPE_MDR,
                                                                                        ALERT_SOURCE_TYPE_APP)
    formatted_data['bar_horizontal_asset_mdr_monthly'] = plot_monthly_horizontal_bars(data, ALERT_TYPE_MDR,
                                                                                      ALERT_SOURCE_TYPE_DEV)

    logger.info(f"[{org_key}] Extract environment configuration and industry comparison in the monthly report")

    # ENVIRONMENT CONFIGURATION
    config_data = get_fixed_values(data)
    formatted_data['mdr_quarantine_action'] = config_data['mdr_quarantine']
    formatted_data['mdr_policy_action'] = config_data['mdr_policy']
    formatted_data['alert_notification_subscription_recipient'] = config_data['alerts_enabled_count']
    formatted_data['daily_report_subscription_recipient'] = config_data['daily_summary_enabled_count']
    formatted_data['monthly_report_subscription_recipient'] = config_data['monthly_report_enabled_count']

    # INDUSTRY COMPARISON
    formatted_data['industry'] = config_data['account_industry']
    formatted_data['industry_size'] = config_data['org_size']
    formatted_data['customer_average'] = config_data['alert_per_asset']
    formatted_data['size_average'] = config_data['size_alert_per_asset']
    formatted_data['industry_average'] = config_data['industry_alerts_per_asset']

    logger.info(f"[{org_key}] Generating asset and licensing plot in the monthly report")

    # ASSETS & LICENSING
    all_asset_data, avg_assets = get_asset_stack_plot_data(data)
    formatted_data['asset_stack_area_chart'] = plot_asset_stack_area_chart(all_asset_data, org_key)
    formatted_data['windows_assets'] = avg_assets['windows']
    formatted_data['macos_assets'] = avg_assets['macos']
    formatted_data['linux_assets'] = avg_assets['linux']

    # formatted_data['area_chart_half_yearly'] = create_area_plot(get_stack_plot_data(data))
    formatted_data['multi_line_chart_half_yearly'] = plot_alert_asset_industry_comp_line_plots()

    template = J2.get_template('templates/monthlyReport/monthlyReport.html')
    html = template.render(formatted_data)
    html_file = f"{org_key}_monthly_report_generated.html"
    with open(html_file, "w") as file_html:
        # Adding the input data to the HTML file
        file_html.write(html)
    return html_file


def get_monthly_report_key(org_key):
    report_year, report_month = get_report_year_month()
    return f'monthly_report/{org_key}/{report_year}-{report_month}.pdf'
