import datetime
import os
from collections import defaultdict

from src.constants import ALERT_TTR
from src.util.logger import logger

AWS_REGION = os.getenv("AWS_REGION", "us-east-1")
MIN_SEVERITY = 5
MAX_SEVERITY = 10


class DailyDataProcessError(Exception):
    def __init__(self, message):
        super().__init__(message)


def get_start_and_end_date():
    """Getting start and date in the report title"""
    report_end_time = datetime.datetime.today() - datetime.timedelta(days=1)
    report_start_time = report_end_time - datetime.timedelta(days=1)
    report_end_time = report_end_time.strftime('%B %d, %Y')
    report_start_time = report_start_time.strftime('%B %d, %Y')
    return report_start_time, report_end_time


def get_time_range_in_plot():
    date_obj = datetime.datetime.today() - datetime.timedelta(days=1)
    date_obj = date_obj.strftime('%b%d%Y')
    month = date_obj[:3]
    day = date_obj[3:5]
    year = date_obj[5:]
    range_str = str(month) + ' ' + str(int(day) - 6) + ' - ' + str(int(day)) + ' ' + str(year)
    return range_str


def get_pie_severity_scores(data, alert_type, date_str):
    """
    Get severities for alerts on specific date
    :param data: raw statistic data
    :param alert_type: 'likely' or 'mdr' alert
    :param date_str: report date in YYYY-MM-DD
    :return: dict with severity as key, counter as value
    """
    try:
        for dct in data["statistics"]:
            if dct["create_date"] == date_str:
                key = alert_type + "_alert_"
                alert_5_count = int(dct[key + "5"]) if dct[key + "5"] else 0
                alert_6_count = int(dct[key + "6"]) if dct[key + "6"] else 0
                alert_7_count = int(dct[key + "7"]) if dct[key + "7"] else 0
                alert_8_count = int(dct[key + "8"]) if dct[key + "8"] else 0
                alert_9_count = int(dct[key + "9"]) if dct[key + "9"] else 0
                alert_10_count = int(dct[key + "10"]) if dct[key + "10"] else 0
                return {'5': alert_5_count,
                        '6': alert_6_count,
                        '7': alert_7_count,
                        '8': alert_8_count,
                        '9': alert_9_count,
                        '10': alert_10_count}
        return {}
    except Exception as e:
        logger.exception(e)
        raise DailyDataProcessError(f'Failed to process data for [{alert_type}] severity pie chart on {date_str}')


def extract_top_app_dev(data, alert_type, data_type, date_str):
    """
    Extract top K process/asset count and name from raw statistic data.
    :param data: raw statistic data
    :param alert_type: 'likely' or 'mdr' alert
    :param data_type: 'process' or 'asset'
    :param date_str: string representation of today in YYYY-MM-DD format
    :return: 2D array [[names of process/asset], [count of each process/asset]]
    """
    try:
        for dct in data["statistics"]:
            if dct["create_date"] == date_str:
                list = []
                for k in range(1, 7):
                    pair = []
                    key = alert_type + '_' + data_type + '_' + str(k)
                    if not (dct[key] == "" or not dct[key]) and not (dct[key + '_cnt'] == "" or not dct[key + '_cnt']):
                        pair.append(dct[key])
                        pair.append(dct[key + '_cnt'])
                        list.append(pair)
                names = [x[0] for x in list]
                count = [int(x[1]) for x in list]
                return names, count
        return [], []
    except Exception as e:
        logger.exception(e)
        raise DailyDataProcessError(f'Failed to top {data_type} for {alert_type} process horizontal bar chart')


def get_line_chart(data, key):
    """
    Get values for specific key for plotting line chart
    :param data: raw statistic data
    :param key: one of 'likely_count', likely_avg_ttr', 'mdr_count', mdr_avg_ttr'
    :return:
    """
    try:
        ls = []
        for dct in data["statistics"]:
            date_value = [datetime.datetime.strptime(dct["create_date"], "%Y-%m-%d").date().strftime("%b %-d\n%Y"), dct[key]]
            ls.append(date_value)
        ls.sort(key=lambda date: datetime.datetime.strptime(date[0], "%b %d %Y"))
        dates = [x[0] for x in ls]
        if 'ttr' in key:
            values = [round(float(x[1]), 1) for x in ls]
        else:
            values = [int(x[1]) for x in ls]
        return [dates, values]
    except Exception as e:
        logger.exception(e)
        raise DailyDataProcessError(f'Failed to process data for line chart with key {key}')


def extract_weekly_severity_bars(data, alert_type):
    """
    Extract weekly counts of alerts with severity between 5 and 10
    :param data: raw statistic data
    :param alert_type: 'likely' or 'mdr' alert
    :return:
    """
    try:
        severity_keys = [f'{alert_type}_alert_{severity}' for severity in range(MIN_SEVERITY, MAX_SEVERITY + 1)]
        daily_severities = defaultdict(list)
        for daily_data in data["statistics"]:
            for severity_key in severity_keys:
                # Add counts of alert by given severity key
                daily_severities[daily_data["create_date"]].append(
                    0 if not daily_data[severity_key] else int(daily_data[severity_key]))
        sorted_daily_severities = {key: daily_severities[key] for key in sorted(daily_severities)}
        # Convert severities grouped by days to severities grouped by values
        severity_counts = [list(pair) for pair in zip(*sorted_daily_severities.values())]
        available_dates = [datetime.datetime.strptime(create_date, "%Y-%m-%d").date().strftime("%b %-d\n%Y") for create_date
                           in sorted_daily_severities.keys()]
        return available_dates, severity_counts
    except Exception as e:
        logger.exception(e)
        raise DailyDataProcessError(f'Failed to process weekly severity bars for {alert_type}')


def get_statistic(data, alert_type, data_type, date_str):
    """
    Extract specific statistic data by key
    :param data_type: 'count' or 'ttr'
    :param alert_type: 'likely' or 'mdr'
    :param data: raw daily data
    :param key: interested key
    :param date_str: report created date in YYYY-MM-DD
    :return: value of statistic key
    """
    if data_type == ALERT_TTR:
        data_type = 'avg_ttr'
    key = f'{alert_type}_{data_type}'
    for dct in data["statistics"]:
        if "create_date" in dct and dct["create_date"] == date_str:
            try:
                value_int = int(dct[key])
                return f'{value_int: ,}'
            except ValueError:
                value_float = round(float(dct[key]))
                return f'{value_float: ,}'
    raise DailyDataProcessError(f'Failed to find the key {key} on {date_str}')
