import datetime
import os
from calendar import monthrange
from collections import defaultdict

from src.util.logger import logger

AWS_REGION = os.getenv("AWS_REGION", "us-east-1")
TOP_APPS_DEVS = 6
MIN_SEVERITY = 5
MAX_SEVERITY = 10


class MonthlyDataProcessError(Exception):
    def __init__(self, message):
        super().__init__(message)


def is_date_in_range(date_check):
    """Checking if date is in a given range"""
    report_year, report_month = get_report_year_month()
    start = datetime.datetime(day=1, month=report_month, year=report_year)
    end = datetime.datetime(day=monthrange(report_year, report_month)[1], month=report_month, year=report_year)
    if start <= datetime.datetime.strptime(date_check, "%Y-%m-%d") <= end:
        return True
    else:
        return False


def get_report_year_month():
    """Get month and year of the monthly report"""
    today = datetime.date.today()
    first = today.replace(day=1)
    prev_date = first - datetime.timedelta(days=1)
    return [prev_date.year, prev_date.month]


def get_num_threats(data, alert_type):
    """Get number of threats for last month"""
    total_threats = 0
    counter = 0  # To differentiate between data being empty vs total threats being 0
    for dct in data["statistics"]:
        counter += 1
        if is_date_in_range(dct['create_date']):
            total_threats += int(dct[alert_type + '_count'])
    if counter > 0:
        return total_threats
    else:
        return '--'


def get_avg_time_to_determination(data, alert_type):
    """Get avg time to determination for last month"""
    total_avg_det_time_sec = 0
    counter = 0
    for dct in data["statistics"]:
        if is_date_in_range(dct['create_date']):
            counter += 1
            total_avg_det_time_sec += float(dct[alert_type + '_avg_ttr'])
    if counter != 0:
        return round(total_avg_det_time_sec / counter / 60, 1)
    else:
        return '--'


def get_start_and_end_date():
    """Getting start and date in the report title"""
    report_year, report_month = get_report_year_month()
    start_date = datetime.date(report_year, report_month, 1)
    end_date = datetime.date(report_year, report_month, monthrange(report_year, report_month)[1])
    start_date = start_date.strftime('%B %d, %Y 2:00am UTC')
    end_date = end_date.strftime('%B %d, %Y 2:00am UTC')
    dates_list = [start_date, end_date]
    return dates_list


def get_plot_date_range():
    """Getting date range in the plots"""
    report_year, report_month = get_report_year_month()
    end_date = datetime.date(report_year, report_month, monthrange(report_year, report_month)[1])
    month = end_date.strftime('%b')
    return month + ' 1-' + end_date.strftime('%d')


def get_dates_of_month(year, month):
    # Calculate the first day of the specified month
    first_day = datetime.datetime(year, month, 1)
    # Calculate the last day of the specified month
    last_day = datetime.datetime(year, month + 1, 1) - datetime.timedelta(days=1)
    # Create an empty list to store date strings
    dates_list = []
    # Iterate over each day in the specified month and add to list
    current_day = first_day
    while current_day <= last_day:
        dates_list.append(current_day.strftime('%Y-%m-%d'))
        current_day += datetime.timedelta(days=1)

    return dates_list


# Getting numerical data in the monthly report plots
def get_fixed_values(data):
    """
    Extract environment config and industry comparison values for the last date of last month.
    :param data:
    :return:
    """
    if data['statistics'] and data['statistics'][-1]:
        last_day_config = data['statistics'][-1]
        try:
            return {
                'mdr_quarantine': last_day_config['mdr_quarantine'],
                'mdr_policy': last_day_config['mdr_policy'],
                'alerts_enabled_count': last_day_config['alerts_enabled_count'],
                'daily_summary_enabled_count': last_day_config['daily_summary_enabled_count'],
                'monthly_report_enabled_count': last_day_config['monthly_report_enabled_count'],
                'account_industry': last_day_config['account_industry'],
                'org_size': last_day_config['org_size'],
                'alert_per_asset': round(float(last_day_config['alert_per_asset']), 1),
                'size_alert_per_asset': round(float(last_day_config['size_alert_per_asset']), 1),
                'industry_alerts_per_asset': round(float(last_day_config['industry_alerts_per_asset']), 1),

            }
        except Exception as e:
            logger.exception(e)
            return ['--'] * 10
            # TODO: Once the query for fetching these values has been verified, raise exception if missing.
            # raise MonthlyDataProcessError(f'Failed to process configuration data {last_day_config}')
    else:
        org_key = data.get("org_key")
        logger.error(f'[{org_key}] Unable to get_fixed_values. Not available in monthly statistic')
        return ['--'] * 10


def get_monthly_line_plot_data(data, statistic_type):
    """
    Extract line data for available dates
    :param data: raw statistic data
    :param statistic_type: one of 'likely_count', likely_avg_ttr', 'mdr_count', mdr_avg_ttr'
    :return: [[start_date, end_date], [values for each date]]
    """
    try:
        available_dates = {}
        for dct in data["statistics"]:
            if 'ttr' in statistic_type:
                available_dates[dct["create_date"]] = round(float(dct[statistic_type]), 1)
            else:
                available_dates[dct["create_date"]] = int(dct[statistic_type])

        if not available_dates:
            org_key = data.get("org_key")
            logger.info(f'[{org_key}] {statistic_type} is not available in monthly report.')
            return [[], []]
        final_lst = []
        report_year, report_month = get_report_year_month()
        for date in get_dates_of_month(report_year, report_month):
            if date in available_dates:
                final_lst.append(available_dates[date])
            else:
                final_lst.append(0)
        start_end_date = [datetime.date(report_year, report_month, 1).strftime("%b %d %Y"),
                          datetime.date(report_year, report_month, monthrange(report_year, report_month)[1]).strftime(
                              "%b %d %Y")]
        return [start_end_date, final_lst]
    except Exception as e:
        logger.exception(e)
        raise MonthlyDataProcessError(f'Failed to process {statistic_type} line chart in monthly report')


def extract_top_app_dev(data, alert_type, data_type):
    """
    Extract top 6 process/asset count for past 30 days
    :param data: raw statistic data
    :param alert_type: 'likely' or 'mdr'
    :param data_type: 'app' or 'dev'
    :return: names of top 6 process/asset, count of top 6 process/asset
    """
    try:
        counter_aggr = defaultdict(int)
        for k in range(1, TOP_APPS_DEVS + 1):
            for dct in data["statistics"]:
                key = alert_type + '_' + data_type + '_' + str(k)
                if dct.get(key):
                    counter_aggr[dct[key]] += int(dct[key + '_cnt'])
        if not counter_aggr:
            org_key = data.get("org_key")
            logger.info(f'[{org_key}] Top {data_type} for {alert_type} is not available in monthly report.')
            return [], []
        else:
            top_items = sorted(counter_aggr.items(), key=lambda item: item[1], reverse=True)[:TOP_APPS_DEVS]
            return [item[0] for item in top_items], [item[1] for item in top_items]
    except Exception as e:
        logger.exception(e)
        raise MonthlyDataProcessError(f'Failed to top {data_type} for {alert_type} process horizontal bar chart')


def get_pie_severity_scores(data, alert_type):
    """
    Aggregate count of alert for each severity in the last month
    :param data: raw statistic data
    :param alert_type: 'likely' or 'mdr' alert
    :return: dict with severity as key and counter as value
    """
    try:
        severity_counter = defaultdict(int)
        for severity in range(MIN_SEVERITY, MAX_SEVERITY + 1):
            for dct in data["statistics"]:
                key = alert_type + '_alert_' + str(severity)
                severity_counter[str(severity)] += int(dct[key]) if dct[key] else 0
        return severity_counter
    except Exception as e:
        logger.exception(e)
        raise MonthlyDataProcessError('Failed to process data for severity pie chart')


def get_asset_stack_plot_data(data):
    """
    Extract count of assets for each OS for last month
    :param data: raw statistic data
    :return: all_asset_dates: Count of assets for each OS type each day
    avg_assets: Average number of asset in last month for each OS type
    """
    try:
        available_dates = {}
        total_win, total_mac, total_linux = 0, 0, 0
        for dct in data["statistics"]:
            windows_cnt = int(dct['registered_active_windows'])
            mac_cnt = int(dct['registered_active_mac'])
            linux_cnt = int(dct['registered_active_linux'])
            total_win += windows_cnt
            total_mac += mac_cnt
            total_linux += linux_cnt
            available_dates[dct["create_date"]] = [windows_cnt, mac_cnt, linux_cnt]

        if not available_dates:
            return [[], []], [0, 0, 0]
        all_asset_dates = {}
        report_year, report_month = get_report_year_month()
        for date in get_dates_of_month(report_year, report_month):
            if date in available_dates:
                all_asset_dates[date] = available_dates[date]
            else:
                all_asset_dates[date] = [0, 0, 0]
        total_asset_dates = len(available_dates)
        avg_assets = {
            'windows': total_win / total_asset_dates,
            'macos': total_mac / total_asset_dates,
            'linux': total_linux / total_asset_dates
        }
        return all_asset_dates, avg_assets
    except Exception as e:
        logger.exception(e)
        raise MonthlyDataProcessError('Failed to process date for asset stack chart')
