import json
import os

import boto3
from botocore.exceptions import ClientError
from pyppeteer import launch

from src.util.logger import logger


class HTMLToPDFConversionError(Exception):
    def __init__(self, message):
        super().__init__(message)


async def generate_pdf_from_html(html_name, pdf_path):
    try:
        browser = await launch(
            headless=True,
            executablePath='/usr/lib/chromium-browser/headless_shell',
            args=[
                '--no-sandbox',
                '--single-process',
                '--disable-dev-shm-usage',
                '--disable-gpu',
                '--no-zygote'
            ]
        )
        page = await browser.newPage()
        logger.debug(f'file://{os.getcwd()}/{html_name}')

        await page.goto(f'file://{os.getcwd()}/{html_name}', waitUntil='networkidle0')

        logger.debug('set content finished')

        await page.pdf({'path': pdf_path})

        await browser.close()
    except Exception as e:
        logger.exception(e)
        raise HTMLToPDFConversionError(message=f'Failed to convert html to pdf {e}')


def from_s3(alert_obj_key, bucket, endpoint='https://s3.us-east-1.amazonaws.com'):
    logger.info(f'Retrieving alert object {alert_obj_key} from {bucket} from {endpoint}')
    s3_client = boto3.client('s3', endpoint_url=endpoint)
    try:
        obj = s3_client.get_object(Bucket=bucket, Key=alert_obj_key)
        body = obj.get('Body').read().decode('utf-8')
        return json.loads(body)
    except ClientError as exc:
        logger.exception(f'failed to retrieve {bucket}/{alert_obj_key} from {endpoint}: {exc}')
        raise exc


def put_to_s3(filepath, s3_key, bucket, endpoint='https://s3.us-east-1.amazonaws.com'):
    """
    Upload PDF file to S3 bucket
    :param filepath: local file path
    :param s3_key: target S3 object key
    :param bucket: target S3 bucket
    :param endpoint:
    :return:
    """
    try:
        logger.info(f'Uploading {filepath} to {bucket}/{s3_key}')
        s3 = boto3.resource('s3', endpoint_url=endpoint)
        obj = s3.Object(bucket, s3_key)
        obj.upload_file(Filename=filepath)
        s3_url = f"https://{bucket}.s3.amazonaws.com/{s3_key}"
        logger.debug(f's3_url = {s3_url}')
        return s3_url
    except ClientError as exc:
        logger.exception(f'failed to upload {bucket}/{s3_key} to {endpoint}: {exc}')
        raise exc
