from src.processor.daily_data_processor import get_pie_severity_scores, get_line_chart, extract_weekly_severity_bars, \
    extract_top_app_dev
from src.widgets.widget_utils import *

from ..util.logger import logger

"""
Generate SVG widgets used in report

"""


class DailyPlotError(Exception):
    def __init__(self, message):
        super().__init__(message)


def plot_daily_line_chart(data, alert_type, data_type):
    """
    Plot line chart for likely/mdr alerts' count/avg TTR (time to respond)
    :param data: raw statistic data
    :param alert_type: 'likely' or 'mdr' alert
    :param data_type: 'count' or 'ttr'
    :return:
    """
    org_key = data.get('org_key', '')
    if 'ttr' in data_type:
        data_type = 'avg_ttr'
    line_data = get_line_chart(data, f'{alert_type}_{data_type}')
    try:
        fig = plt.figure(figsize=(80, 25))
        if not line_data[0]:
            return 'src/default_image.png'
        ax = plt.gca()
        ax.plot(line_data[1], lw=7, color='mediumorchid')
        ax.spines['top'].set_visible(False)
        ax.spines['right'].set_visible(False)
        ax.grid(axis='y', color="dimgray", linestyle='dashed', alpha=1, linewidth=0.9, dashes=(50, 50))
        ax.set_xticks(ticks=np.arange(len(line_data[0])), labels=line_data[0])
        ax.legend().remove()
        ax.tick_params(axis='x', which='major', labelsize=80, left=False, bottom=False)
        ax.tick_params(axis='y', labelsize=70, left=False)
        ax.set(xlabel=None)
        plot_name = f'{org_key}_daily_line_{alert_type}_{data_type}.svg'
        fig.savefig(plot_name, format='svg', bbox_inches='tight')
        plt.close()
        return plot_name
    except Exception as e:
        logger.exception(e)
        raise DailyPlotError(f'Failed to plot line chart for {alert_type}_{data_type} with {line_data}')


def plot_daily_horizontal_bars(data, alert_type, alert_source_type, date_str):
    """
    Create horizontal bars for alert/asset counts
    :param data: daily statistic data
    :param alert_type: 'likely' or 'mdr' alert
    :param alert_source_type: 'process' or 'asset'
    :param date_str: string representation of today in YYYY-MM-DD format
    :return:
    """
    names, counts = extract_top_app_dev(data, alert_type, alert_source_type, date_str)
    try:
        return plot_horizontal_bars(names, counts, alert_type, alert_source_type,
                                    report_type='daily', org_key=data.get('org_key'))
    except Exception as e:
        logger.exception(e)
        raise DailyPlotError(f'Failed to plot horizontal bars {alert_type}_{alert_source_type} with data {names} {counts}')


def plot_daily_severity_pie_chart(data, alert_type, date_str):
    """
    Plot pie chart of alert severity distributions today.
    :param data: raw statistic data
    :param alert_type: 'likely' or 'mdr' alert
    :param date_str: daily report generated date in YYYY-MM-DD
    :return:
    """
    daily_pie_data = get_pie_severity_scores(data, alert_type, date_str)
    try:
        return plot_pie_severity(daily_pie_data, alert_type, report_type='daily', org_key=data.get('org_key'))
    except Exception as e:
        logger.exception(e)
        raise DailyPlotError(f'Failed to plot [{alert_type}] severity pie chart with {daily_pie_data} on {date_str}')


def plot_weekly_severity_bars(data, alert_type, org_key):
    """
    Plot severity bars for past 7 days
    :param org_key:
    :param data: raw statistic data
    :param alert_type: 'likely' or 'mdr' alert
    :return: path of svg file
    """
    available_dates, severity_counts = extract_weekly_severity_bars(data, alert_type)
    try:
        fig = plt.figure(figsize=(80, 25))
        if not severity_counts:
            return 'src/default_image.png'
        ax = plt.gca()
        width = 0.1
        severity_5 = severity_counts[0]
        severity_6 = severity_counts[1]
        severity_7 = severity_counts[2]
        severity_8 = severity_counts[3]
        severity_9 = severity_counts[4]
        severity_10 = severity_counts[5]
        x = np.arange(len(available_dates))
        date = available_dates

        plt.bar(x, severity_10, width=0.1, color=SEVERITY_10)
        plt.bar(x + width, severity_9, width=0.1, color=SEVERITY_9)
        plt.bar(x + width * 2, severity_8, width=0.1, color=SEVERITY_8)
        plt.bar(x + width * 3, severity_7, width=0.1, color=SEVERITY_7)
        plt.bar(x + width * 4, severity_6, width=0.1, color=SEVERITY_6)
        plt.bar(x + width * 5, severity_5, width=0.1, color=SEVERITY_5)

        plt.xticks(ticks=x + width, labels=date)
        my_labels = ["10 (Priority)", "9 (Priority)", "8 (Priority)", "7 (Eligible)", "6 (Eligible)", "5 (Eligible)"]
        ax.legend(loc="lower center", ncol=9, bbox_to_anchor=(0.5, -0.4), frameon=False, markerfirst=False,
                  labels=my_labels, fontsize=60)
        ax.spines['top'].set_visible(False)
        ax.spines['right'].set_visible(False)
        ax.grid(axis='y', color="dimgray", linestyle='dashed', alpha=1, linewidth=0.9, dashes=(50, 50))
        ax.set(xlabel=None)
        ax.tick_params(axis='x', which='major', labelsize=60, left=False, bottom=False)
        ax.tick_params(axis='y', labelsize=60, left=False)
        fig.subplots_adjust(bottom=0.3)
        plot_name = f'{org_key}_daily_bar_chart_severity_{alert_type}.svg'
        fig.savefig(plot_name, format='svg', bbox_inches='tight')
        plt.close()
        return plot_name
    except Exception as e:
        logger.exception(e)
        raise DailyPlotError(f'Failed to plot weekly severity bars for {severity_counts} on {available_dates}')


