from datetime import datetime
from itertools import cycle, islice

import matplotlib.pyplot as plt

from src.processor.monthly_data_processor import get_pie_severity_scores, get_monthly_line_plot_data, \
    extract_top_app_dev
from src.widgets.widget_utils import plot_horizontal_bars, plot_pie_severity
from ..util.logger import logger


class MonthlyPlotError(Exception):
    def __init__(self, message):
        super().__init__(message)


def plot_monthly_line_chart(data, alert_type, data_type):
    """
    Plot line chart for likely/mdr alerts' count/avg TTR (time to respond)
    :param data: raw statistic data
    :param alert_type: 'likely' or 'mdr' alert
    :param data_type: 'count' or 'ttr'
    :return: SVG file path
    """
    if 'ttr' in data_type:
        data_type = 'avg_ttr'
    line_data = get_monthly_line_plot_data(data, f'{alert_type}_{data_type}')
    org_key = data.get("org_key")
    try:
        if not line_data[0]:
            return 'src/default_image.png'
        fig = plt.figure(figsize=(40, 6))
        ax = plt.gca()
        y = line_data[1]
        ax.plot(y, lw=4, color='mediumpurple')
        ax.spines['top'].set_visible(False)
        ax.spines['right'].set_visible(False)
        ax.grid(axis='y', color="dimgray", linestyle='dashed', alpha=0.8, linewidth=1, dashes=(50, 50))
        ax.xaxis.set_major_locator(plt.MaxNLocator(2))
        ax.set_xticks(ticks=[0, 30], labels=[line_data[0][0], line_data[0][1]])
        ax.legend().remove()
        ax.tick_params(axis='x', which='major', labelsize=22, left=False, bottom=False)
        ax.tick_params(axis='y', labelsize=30, left=False)
        ax.set(xlabel=None)
        plot_name = f'{org_key}_monthly_line_chart_{alert_type}_{data_type}.svg'
        fig.savefig(plot_name, format='svg', bbox_inches='tight')
        return plot_name
    except Exception as e:
        logger.exception(e)
        raise MonthlyPlotError(f'Failed to plot line chat for [{alert_type}_{data_type}] with data {line_data}')


def plot_monthly_horizontal_bars(data, alert_type, data_type):
    """
    Extract top 6 process/asset count for past 30 days
    :param data: raw statistic data
    :param alert_type: 'likely' or 'mdr'
    :param data_type: 'app' or 'dev'
    :return: SVG file path
    """
    names, counts = extract_top_app_dev(data, alert_type, data_type)
    try:
        return plot_horizontal_bars(names, counts, alert_type, data_type,
                                    report_type='monthly', org_key=data.get('org_key'))
    except Exception as e:
        logger.exception(e)
        raise MonthlyPlotError(f'Failed to plot horizontal bars {alert_type}_{data_type} with data {names} {counts}')


def plot_monthly_severity_pie_chart(data, alert_type):
    """
    Plot pie chart of alert severity distributions today.
    :param data: raw statistic data
    :param alert_type: 'likely' or 'mdr' alert
    :return: SVG file path
    """
    monthly_severities = get_pie_severity_scores(data, alert_type=alert_type)
    try:
        return plot_pie_severity(monthly_severities, alert_type, report_type='monthly', org_key=data.get('org_key'))
    except Exception as e:
        logger.exception(e)
        raise MonthlyPlotError(f'Failed to plot [{alert_type}] severity pie chart with {monthly_severities}')


def plot_asset_stack_area_chart(all_asset_data, org_key):
    """
    Plot stack area chart for asset by OS
    :param org_key:
    :param all_asset_data: Dict with date as key and list of values as count for windows, mac, and linux OS
    :return:
    """
    try:
        fig = plt.figure(figsize=(30, 12))
        ax = plt.gca()
        sorted_dates = sorted(all_asset_data.keys())
        windows = [all_asset_data[date][0] for date in sorted_dates]
        mac = [all_asset_data[date][1] for date in sorted_dates]
        linux = [all_asset_data[date][2] for date in sorted_dates]

        # Convert sorted_dates to datetime objects for plotting
        dates = [datetime.strptime(date, '%Y-%m-%d').strftime('%b %d') for date in sorted_dates]
        ax.stackplot(dates, windows, mac, linux,
                     colors=list(islice(cycle(['royalblue', 'lightskyblue', 'mediumorchid']),
                                        None, len(sorted_dates))))
        my_labels = ["Windows Assets", "MacOS Assets", "Linux Assets"]
        ax.legend(loc="lower left", ncol=3, bbox_to_anchor=(0.3, -0.22), frameon=False, labels=my_labels,
                  fontsize='large',
                  handleheight=4, markerfirst=False, handlelength=6)
        ax.spines['top'].set_visible(False)
        ax.spines['right'].set_visible(False)
        ax.grid(axis='y', color="grey", linestyle='dashed', alpha=1, linewidth=0.9, dashes=(50, 50))
        ax.tick_params(axis='x', which='major', labelsize=16, left=False, bottom=False)
        ax.tick_params(axis='y', labelsize=16, left=False)
        plt.xticks(rotation=45, ha='right')
        ax.text(x=1.5, y=-25, s="Operating System", fontsize='x-large', fontweight='bold', fontstyle='italic')
        ax.set(xlabel=None)
        fig.subplots_adjust(bottom=0.3)
        plot_name = f'{org_key}_monthly_asset_stack_area_chart.svg'
        fig.savefig(plot_name, format='svg', bbox_inches='tight')
        return plot_name
    except Exception as e:
        logger.exception(e)
        raise MonthlyPlotError(f'Failed to plot asset stack chart with data {all_asset_data}')


def plot_half_year_severities():
    # TODO: Create widgets after processing data for this chart
    return 'src/default_image.png'


def plot_half_year_line_chart():
    # TODO: Create widgets after processing data for this chart
    return 'src/default_image.png'


def plot_alert_asset_industry_comp_line_plots():
    # TODO: Create widgets after processing data for this chart
    return 'src/default_image.png'
